#!/bin/sh
############################################################################
#                                                                          #
#  Copyright (c) 2001, 2011 NoMachine, http://www.nomachine.com.           #
#                                                                          #
#  NXSERVER, NX protocol compression and NX extensions to this software    #
#  are copyright of NoMachine. Redistribution and use of the present       #
#  software is allowed according to terms specified in the file LICENSE    #
#  which comes in the source distribution.                                 #
#                                                                          #
#  Check http://www.nomachine.com/licensing.html for applicability.        #
#                                                                          #
#  NX and NoMachine are trademarks of Medialogic S.p.A.                    #
#                                                                          #
#  All rights reserved.                                                    #
#                                                                          #
############################################################################
PATH="/usr/bin:/bin:/usr/local/bin" && export PATH

NODE_ROOT='/usr/NX'

CONFIG_FILE="$NODE_ROOT/etc/server.cfg"
#CONFIG_FILE=$1

COMMAND_ECHO="echo"
COMMAND_STAT="stat"
COMMAND_FLOCK="flock"
COMMAND_SED="sed"
COMMAND_CAT="/bin/cat"

if [ -x /bin/rm ];
then
  RMCOMMAND="/bin/rm"
else 
  RMCOMMAND="rm"
fi

if [ -x /bin/cp ];
then
  CPCOMMAND="/bin/cp"
else
  CPCOMMAND="cp"
fi

if [ -x /bin/mv ];
then
  MVCOMMAND="/bin/mv"
else
  MVCOMMAND="mv"
fi

if [ -e "/bin/awk" ];
then
  COMMAND_AWK="/bin/awk"
elif [ -e "/usr/bin/awk" ];
then
  COMMAND_AWK="/usr/bin/awk"
else
  echo "ERROR: awk command not found"
  exit 1
fi

if [ "x$1" != "xnx" -a "x$1" != "xroot" ];
  then
    $COMMAND_ECHO "NX> 599 Error: nxconfigure.sh: only 'nx' user can change configuration key by this script" 1>&2
    exit 1
fi

checkPpid() {

  if [ "x$1" = "x" ];
  then
    $COMMAND_ECHO "Missing pid"
    return 1
  fi
  if [ ! -e /proc/$1/cmdline ];
  then
    $COMMAND_ECHO "Wrong pid: $1"
    return 1
  fi

  command=$($COMMAND_CAT /proc/$1/cmdline)

  $COMMAND_ECHO "$command"
}

command=$(checkPpid $2)

if [ $? != 0 ];
then
  $COMMAND_ECHO "NX> 500 Error: $command"
  exit 1
fi

if [ "$command" != "nxserver-c$NODE_ROOT/bin/nxserver --login" ];
then
  $COMMAND_ECHO "NX> 500 Error: Only the nxserver can run this script: $command.\n";
  exit 1
fi

  if [ ! -r "$CONFIG_FILE" ];
  then
    $COMMAND_ECHO "Error: Unable to read file '$CONFIG_FILE'" 1>&2
    exit 1
  fi

  if [ ! -f "$CONFIG_FILE.backup" ];
  then
    command="$CPCOMMAND -f '$CONFIG_FILE' '$CONFIG_FILE.backup'"
    result=`/bin/bash -c "$command" 2>&1`
  else
    command="$RMCOMMAND -f '$CONFIG_FILE.backup'"
    result=`/bin/bash -c "$command" 2>&1`
    if [ $? != 0 ] || [ x$result != x ];
    then
      $COMMAND_ECHO "ERROR: Problems while removing: '$CONFIG_FILE.backup' file: [$!][$result]" 1>&2
      $COMMAND_ECHO "NX> 500 Error: Problems while removing '$CONFIG_FILE.backup' file: [$!][$result]"
      exit $?
    else
      command="$CPCOMMAND -f '$CONFIG_FILE' '$CONFIG_FILE.backup'"
      result=`/bin/bash -c "$command" 2>&1`
    fi
  fi

  if [ $? != 0 ] || [ x$result != x ];
  then
    $COMMAND_ECHO "ERROR: Problems while coping: '$CONFIG_FILE' to '$CONFIG_FILE.backup': [$!][$result]" 1>&2
    $COMMAND_ECHO "NX> 500 Error: Problems while coping: '$CONFIG_FILE' to '$CONFIG_FILE.backup': [$!][$result]"
    exit $?
  else
    $COMMAND_ECHO "NX> 900 File:$CONFIG_FILE copied in $CONFIG_FILE.backup with no error result:[$!][$result] exiting with:[$?]"
  fi


  
  EXPRESSION="^ *\#* *$3 *=.*"
  command="$COMMAND_FLOCK '$CONFIG_FILE' grep '$EXPRESSION' '$CONFIG_FILE' "
  #$COMMAND_ECHO "NX> 900 running command:[$command]"
  result=`/bin/bash -c "$command" 2>&1`
  GREP_RESULT="$?"
  $COMMAND_ECHO "NX> 900 command:[$command] has result:[$result] \$?=[$GREP_RESULT]"
  
 
  if [ "x$GREP_RESULT" = "x0" ];
  then
    #key found, to be updated
    $COMMAND_ECHO "NX> 900 key $3 found, updating... [$?]"
    EXPRESSION="s|^ *\#* *$3 *=.*|$3 = \"$4\"|g"
    command="$COMMAND_FLOCK $CONFIG_FILE $COMMAND_SED -r '$EXPRESSION' $CONFIG_FILE > $CONFIG_FILE.tmp"
    $COMMAND_ECHO "NX> 900 running command:[$command]"
    result=`/bin/bash -c "$command" 2>&1`

    if [ $? != 0 ] || [ x$result != x ];
    then
      $COMMAND_ECHO "ERROR: Problems while updating: '$CONFIG_FILE.tmp' file: [$!][$result]" 1>&2
      $COMMAND_ECHO "NX> 500 Error: Problems while updating: [$!][$result]"
      exit $?
    else
      $COMMAND_ECHO "NX> 900 File:'$CONFIG_FILE.tmp' updated with no error result:[$!][$result] exiting with:[$?]"
    fi

    if [ -f "$CONFIG_FILE.tmp" ];
    then
      chmod 0644 "$CONFIG_FILE.tmp"
      chown root:root "$CONFIG_FILE.tmp"
      $COMMAND_ECHO "NX> 900 moving '$CONFIG_FILE.tmp' in '$CONFIG_FILE'"
      command="$MVCOMMAND -f '$CONFIG_FILE.tmp' '$CONFIG_FILE'"
      result=`/bin/bash -c "$command" 2>&1`
    fi

    if [ $? != 0 ] || [ x$result != x ];
    then
      $COMMAND_ECHO "ERROR: Problems while moving: '$CONFIG_FILE.tmp' file: [$!][$result]" 1>&2
      $COMMAND_ECHO "NX> 500 Error: Problems while moving: '$CONFIG_FILE.tmp' file: [$!][$result]"
      #add here the recovery of backuppped file procedure.
      exit $?
    else
      $COMMAND_ECHO "NX> 900 File:'$CONFIG_FILE.tmp' moved in '$CONFIG_FILE' with no error result:[$!][$result] exiting with:[$?]"
    fi

    if [ -f "$CONFIG_FILE.backup" ];
    then
      $COMMAND_ECHO "NX> 900 removing '$CONFIG_FILE.backup'"
      command="$RMCOMMAND -f '$CONFIG_FILE.backup'"
      result=`/bin/bash -c "$command" 2>&1`
    fi

    if [ $? != 0 ] || [ x$result != x ];
    then
      $COMMAND_ECHO "ERROR: Problems while removing: '$CONFIG_FILE.backup' file: [$!][$result]" 1>&2
      $COMMAND_ECHO "NX> 500 Error: Problems while removing '$CONFIG_FILE.backup' file: [$!][$result]"
      exit $?
    else
      $COMMAND_ECHO "NX> 900 File:'$CONFIG_FILE.backup' removed with no error result:[$!][$result] exiting with:[$?]"
      FINAL_RESULT="substituted"
    fi

  elif [ "x$GREP_RESULT" = "x1" ];
    then
    #key not found, appending the key to the end of file.
    $COMMAND_ECHO "NX> 900 key not found, appending the key to the end of file."
    KEY="$3 = \"$4\""
    command="$COMMAND_FLOCK '$CONFIG_FILE' $COMMAND_ECHO '$KEY' >> '$CONFIG_FILE'"
    $COMMAND_ECHO "NX> 900 running command:[$command]"
    result=`/bin/bash -c "$command" 2>&1`

    if [ $? != 0 ] || [ x$result != x ];
    then
      $COMMAND_ECHO "ERROR: Problems while updating: '$CONFIG_FILE' file: [$!][$result]" 1>&2
      $COMMAND_ECHO "NX> 500 Error: Problems while updating: [$!][$result]"
      exit $?
    else
      $COMMAND_ECHO "NX> 900 File:'$CONFIG_FILE' updated with no error result:[$!][$result] exiting with:[$?]"
      FINAL_RESULT="appended"
    fi
  fi

    $COMMAND_ECHO "NX> 900 key $3 has been $FINAL_RESULT"

exit 0
